"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.dispatchAction = dispatchAction;
exports["default"] = instances;
exports.getActiveInstance = exports.initialState = void 0;

var _actionTypes = require("../constants/actionTypes");

var _socketActionTypes = require("../constants/socketActionTypes");

var _parseJSON = _interopRequireDefault(require("../utils/parseJSON"));

var _updateState = require("../utils/updateState");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && Symbol.iterator in Object(iter)) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var initialState = {
  selected: null,
  current: 'default',
  sync: false,
  connections: {},
  options: {
    "default": {
      features: {}
    }
  },
  states: {
    "default": {
      actionsById: {},
      computedStates: [],
      currentStateIndex: -1,
      nextActionId: 0,
      skippedActionIds: [],
      stagedActionIds: []
    }
  }
};
exports.initialState = initialState;

function updateState(state, request, id, serialize) {
  var payload = request.payload;
  var actionsById = request.actionsById;

  if (actionsById) {
    payload = _objectSpread(_objectSpread({}, payload), {}, {
      actionsById: (0, _parseJSON["default"])(actionsById, serialize),
      computedStates: (0, _parseJSON["default"])(request.computedStates, serialize)
    });

    if (request.type === 'STATE' && request.committedState) {
      payload.committedState = payload.computedStates[0].state;
    }
  } else {
    payload = (0, _parseJSON["default"])(payload, serialize);
  }

  var newState;
  var liftedState = state[id] || state["default"];
  var action = request.action && (0, _parseJSON["default"])(request.action, serialize) || {};

  switch (request.type) {
    case 'INIT':
      newState = (0, _updateState.recompute)(state["default"], payload, {
        action: {
          type: '@@INIT'
        },
        timestamp: action.timestamp || Date.now()
      });
      break;

    case 'ACTION':
      {
        var isExcess = request.isExcess;
        var nextActionId = request.nextActionId || liftedState.nextActionId + 1;
        var maxAge = request.maxAge;

        if (Array.isArray(action)) {
          // Batched actions
          newState = liftedState;

          for (var i = 0; i < action.length; i++) {
            newState = (0, _updateState.recompute)(newState, request.batched ? payload : payload[i], action[i], newState.nextActionId + 1, maxAge, isExcess);
          }
        } else {
          newState = (0, _updateState.recompute)(liftedState, payload, action, nextActionId, maxAge, isExcess);
        }

        break;
      }

    case 'STATE':
      newState = payload;

      if (newState.computedStates.length <= newState.currentStateIndex) {
        newState.currentStateIndex = newState.computedStates.length - 1;
      }

      break;

    case 'PARTIAL_STATE':
      {
        var _maxAge = request.maxAge;
        var _nextActionId = payload.nextActionId;
        var stagedActionIds = payload.stagedActionIds;
        var computedStates = payload.computedStates;
        var oldActionsById;
        var oldComputedStates;
        var committedState;

        if (_nextActionId > _maxAge) {
          var oldStagedActionIds = liftedState.stagedActionIds;
          var excess = oldStagedActionIds.indexOf(stagedActionIds[1]);
          var key;

          if (excess > 0) {
            oldComputedStates = liftedState.computedStates.slice(excess - 1);
            oldActionsById = _objectSpread({}, liftedState.actionsById);

            for (var _i = 1; _i < excess; _i++) {
              key = oldStagedActionIds[_i];
              if (key) delete oldActionsById[key];
            }

            committedState = computedStates[0].state;
          } else {
            oldActionsById = liftedState.actionsById;
            oldComputedStates = liftedState.computedStates;
            committedState = liftedState.committedState;
          }
        } else {
          oldActionsById = liftedState.actionsById;
          oldComputedStates = liftedState.computedStates;
          committedState = liftedState.committedState;
        }

        computedStates = [].concat(_toConsumableArray(oldComputedStates), _toConsumableArray(computedStates));
        var statesCount = computedStates.length;
        var currentStateIndex = payload.currentStateIndex;
        if (statesCount <= currentStateIndex) currentStateIndex = statesCount - 1;
        newState = _objectSpread(_objectSpread({}, liftedState), {}, {
          actionsById: _objectSpread(_objectSpread({}, oldActionsById), payload.actionsById),
          computedStates: computedStates,
          currentStateIndex: currentStateIndex,
          nextActionId: _nextActionId,
          stagedActionIds: stagedActionIds,
          committedState: committedState
        });
        break;
      }

    case 'LIFTED':
      newState = liftedState;
      break;

    default:
      return state;
  }

  if (request.liftedState) newState = _objectSpread(_objectSpread({}, newState), request.liftedState);
  return _objectSpread(_objectSpread({}, state), {}, _defineProperty({}, id, newState));
}

function dispatchAction(state, _ref) {
  var action = _ref.action;

  if (action.type === 'JUMP_TO_STATE' || action.type === 'JUMP_TO_ACTION') {
    var id = state.selected || state.current;
    var liftedState = state.states[id];
    var currentStateIndex = action.index;

    if (typeof currentStateIndex === 'undefined' && action.actionId) {
      currentStateIndex = liftedState.stagedActionIds.indexOf(action.actionId);
    }

    return _objectSpread(_objectSpread({}, state), {}, {
      states: _objectSpread(_objectSpread({}, state.states), {}, _defineProperty({}, id, _objectSpread(_objectSpread({}, liftedState), {}, {
        currentStateIndex: currentStateIndex
      })))
    });
  }

  return state;
}

function removeState(state, connectionId) {
  var instanceIds = state.connections[connectionId];
  if (!instanceIds) return state;

  var connections = _objectSpread({}, state.connections);

  var options = _objectSpread({}, state.options);

  var states = _objectSpread({}, state.states);

  var selected = state.selected;
  var current = state.current;
  var sync = state.sync;
  delete connections[connectionId];
  instanceIds.forEach(function (id) {
    if (id === selected) {
      selected = null;
      sync = false;
    }

    if (id === current) {
      var inst = Object.keys(connections)[0];
      if (inst) current = connections[inst][0];else current = 'default';
    }

    delete options[id];
    delete states[id];
  });
  return {
    selected: selected,
    current: current,
    sync: sync,
    connections: connections,
    options: options,
    states: states
  };
}

function init(_ref2, connectionId, current) {
  var type = _ref2.type,
      action = _ref2.action,
      name = _ref2.name,
      _ref2$libConfig = _ref2.libConfig,
      libConfig = _ref2$libConfig === void 0 ? {} : _ref2$libConfig;
  var lib;
  var actionCreators;
  var creators = libConfig.actionCreators || action;
  if (typeof creators === 'string') creators = JSON.parse(creators);
  if (Array.isArray(creators)) actionCreators = creators;
  if (type === 'STATE') lib = 'redux';
  return {
    name: libConfig.name || name || current,
    connectionId: connectionId,
    explicitLib: libConfig.type,
    lib: lib,
    actionCreators: actionCreators,
    features: libConfig.features ? libConfig.features : {
      lock: lib === 'redux',
      "export": libConfig.type === 'redux' ? 'custom' : true,
      "import": 'custom',
      persist: true,
      pause: true,
      reorder: true,
      jump: true,
      skip: true,
      dispatch: true,
      sync: true,
      test: true
    },
    serialize: libConfig.serialize
  };
}

function instances() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
  var action = arguments.length > 1 ? arguments[1] : undefined;

  switch (action.type) {
    case _actionTypes.UPDATE_STATE:
      {
        var request = action.request;
        if (!request) return state;
        var connectionId = action.id || request.id;
        var current = request.instanceId || connectionId;
        var connections = state.connections;
        var options = state.options;

        if (typeof state.options[current] === 'undefined') {
          connections = _objectSpread(_objectSpread({}, state.connections), {}, _defineProperty({}, connectionId, [].concat(_toConsumableArray(connections[connectionId] || []), [current])));
          options = _objectSpread(_objectSpread({}, options), {}, _defineProperty({}, current, init(request, connectionId, current)));
        }

        return _objectSpread(_objectSpread({}, state), {}, {
          current: current,
          connections: connections,
          options: options,
          states: updateState(state.states, request, current, options[current].serialize)
        });
      }

    case _actionTypes.SET_STATE:
      return _objectSpread(_objectSpread({}, state), {}, {
        states: _objectSpread(_objectSpread({}, state.states), {}, _defineProperty({}, getActiveInstance(state), action.newState))
      });

    case _actionTypes.TOGGLE_PERSIST:
      return _objectSpread(_objectSpread({}, state), {}, {
        persisted: !state.persisted
      });

    case _actionTypes.TOGGLE_SYNC:
      return _objectSpread(_objectSpread({}, state), {}, {
        sync: !state.sync
      });

    case _actionTypes.SELECT_INSTANCE:
      return _objectSpread(_objectSpread({}, state), {}, {
        selected: action.selected,
        sync: false
      });

    case _actionTypes.REMOVE_INSTANCE:
      return removeState(state, action.id);

    case _actionTypes.LIFTED_ACTION:
      {
        if (action.message === 'DISPATCH') return dispatchAction(state, action);

        if (action.message === 'IMPORT') {
          var id = state.selected || state.current;

          if (state.options[id].features["import"] === true) {
            return _objectSpread(_objectSpread({}, state), {}, {
              states: _objectSpread(_objectSpread({}, state.states), {}, _defineProperty({}, id, (0, _parseJSON["default"])(action.state)))
            });
          }
        }

        return state;
      }

    case _socketActionTypes.DISCONNECTED:
      return initialState;

    default:
      return state;
  }
}
/* eslint-disable no-shadow */


var getActiveInstance = function getActiveInstance(instances) {
  return instances.selected || instances.current;
};
/* eslint-enable */


exports.getActiveInstance = getActiveInstance;