"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = configureStore;

var _redux = require("redux");

var _localforage = _interopRequireDefault(require("localforage"));

var _reduxPersist = require("redux-persist");

var _api = _interopRequireDefault(require("../middlewares/api"));

var _exportState = _interopRequireDefault(require("../middlewares/exportState"));

var _reducers = _interopRequireDefault(require("../reducers"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function configureStore(callback, key) {
  var persistConfig = {
    keyPrefix: "redux-devtools".concat(key || '', ":"),
    blacklist: ['instances', 'socket'],
    storage: _localforage["default"],
    serialize: function serialize(data) {
      return data;
    },
    deserialize: function deserialize(data) {
      return data;
    }
  };
  (0, _reduxPersist.getStoredState)(persistConfig, function (err, restoredState) {
    var composeEnhancers = _redux.compose;

    if (process.env.NODE_ENV !== 'production') {
      if (window.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__) {
        composeEnhancers = window.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__;
      }

      if (module.hot) {
        // Enable Webpack hot module replacement for reducers
        module.hot.accept('../reducers', function () {
          var nextReducer = require('../reducers'); // eslint-disable-line global-require


          store.replaceReducer(nextReducer);
        });
      }
    }

    var store = (0, _redux.createStore)(_reducers["default"], restoredState, composeEnhancers((0, _redux.applyMiddleware)(_exportState["default"], _api["default"])));
    var persistor = (0, _reduxPersist.createPersistor)(store, persistConfig);
    callback(store, restoredState);
    if (err) persistor.purge();
  });
}