"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.sweep = sweep;
exports.nonReduxDispatch = nonReduxDispatch;

var _difference = _interopRequireDefault(require("lodash/difference"));

var _omit = _interopRequireDefault(require("lodash/omit"));

var _stringifyJSON = _interopRequireDefault(require("./stringifyJSON"));

var _actionTypes = require("../constants/actionTypes");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function sweep(state) {
  return _objectSpread(_objectSpread({}, state), {}, {
    actionsById: (0, _omit["default"])(state.actionsById, state.skippedActionIds),
    stagedActionIds: (0, _difference["default"])(state.stagedActionIds, state.skippedActionIds),
    skippedActionIds: [],
    currentStateIndex: Math.min(state.currentStateIndex, state.stagedActionIds.length - 1)
  });
}

function nonReduxDispatch(store, message, instanceId, action, initialState, preInstances) {
  var instances = preInstances || store.getState().instances;
  var state = instances.states[instanceId];
  var options = instances.options[instanceId];

  if (message !== 'DISPATCH') {
    if (message === 'IMPORT') {
      if (options.features["import"] === true) {
        return (0, _stringifyJSON["default"])(state.computedStates[state.currentStateIndex].state, true);
      }

      return initialState;
    }

    return undefined;
  }

  if (options.lib === 'redux') return undefined;

  switch (action.type) {
    case 'TOGGLE_ACTION':
      return (0, _stringifyJSON["default"])(state, true);

    case 'JUMP_TO_STATE':
      return (0, _stringifyJSON["default"])(state.computedStates[action.index].state, true);

    case 'JUMP_TO_ACTION':
      return (0, _stringifyJSON["default"])(state.computedStates[state.stagedActionIds.indexOf(action.actionId)].state, true);

    case 'ROLLBACK':
      return (0, _stringifyJSON["default"])(state.computedStates[0].state, true);

    case 'SWEEP':
      store.dispatch({
        type: _actionTypes.SET_STATE,
        newState: sweep(state)
      });
      return undefined;

    default:
      return undefined;
  }
}